Require Import List.
Import ListNotations.
Require Import mtac hash.


Module WithList.

  Definition dyn := { x : Prop | x}.
  Definition Dyn := @exist Prop (fun p=>p).

  Definition ProofNotFound : Exception.
    exact exception.
  Qed.

  Program
  Definition lookup (p : Prop)  := 
    mfix f (s : list dyn) : M p :=
      mmatch s return M p with
      | [x s'] (Dyn p x) :: s' => ret x
      | [d s'] d :: s' => f s'
      | _ => raise ProofNotFound
      end.
  
  Program
  Definition tauto' :=
    mfix f (c : list dyn) (p : Prop) : M p :=
      mmatch p as p' return M p' with
      | True => ret I 
      | [p1 p2] p1 /\ p2 =>
           r1 <- f c p1 ;
           r2 <- f c p2 ;
           ret (conj r1 r2)
      | [p1 p2]  p1 \/ p2 =>
           mtry 
             r1 <- f c p1 ;
             ret (or_introl r1)
           with _ =>
             r2 <- f c p2 ;
             ret (or_intror r2)
           end
      | [p1 p2 : Prop] p1 -> p2 =>
           nu (x:p1),
             r <- f (Dyn p1 x :: c) p2;
             abs x r
      | [A (q:A -> Prop)] (forall x:A, q x) =>
           nu (x:A),
             r <- f c (q x);
             abs x r
      | [A (q:A -> Prop)] (exists x:A, q x) =>
           X <- evar A;
           r <- f c (q X);
           b <- is_evar X;
           if b then 
             raise ProofNotFound
           else
             ret (ex_intro q X r)
      | [p':Prop] p' => lookup p' c
      end.
  
  Definition tauto P := 
    tauto' nil P.

End WithList.

Example ex1 : True /\ (True \/ False) := run (WithList.tauto _).

Module WithMap.
  Require Import FMapPositive.
  Require Import NArith.

  Definition dyn := { x : Prop | x}.
  Definition Dyn := @exist Prop (fun p=>p).

  Definition ProofNotFound : Exception.
    exact exception.
  Qed.

  Definition ctx := PositiveMap.t (list dyn).
  Definition limit := 65535%N.

  (* 0 is mapped to 1, and p is mapped to p *)
  Definition to_pos n :=
    match n with
    | N0 => xH
    | Npos p => p
    end.
  Coercion to_pos : N >-> positive.

  Program
  Definition find  (x : Prop) :=
    mfix f (s : list dyn) : M x :=
      mmatch s return M x with
      | [y s'] (Dyn x y :: s') => ret y
      | [y s'] (y :: s') => f s'
      | _ => raise ProofNotFound
      end.

  Program
  Definition tauto' :=
    mfix f (c : ctx) (p : Prop) : M p :=
      mmatch p as p' return M p' with
      | True => ret I 
      | [p1 p2] p1 /\ p2 =>
           r1 <- f c p1 ;
           r2 <- f c p2 ;
           ret (conj r1 r2)
      | [p1 p2]  p1 \/ p2 =>
           mtry 
             r1 <- f c p1 ;
             ret (or_introl r1)
           with _ =>
             r2 <- f c p2 ;
             ret (or_intror r2)
           end
      | [p1 p2 : Prop] p1 -> p2 =>
           nu (x:p1),
             h <- hash p1 limit;
             l <- retS 
               match PositiveMap.find h c with
                 | None => []
                 | Some l => l
               end;
             c' <- retS (PositiveMap.add h (Dyn p1 x :: l) c);
             r <- f c' p2;
             abs x r
      | [A (q:A -> Prop)] (forall x:A, q x) =>
           nu (x:A),
             r <- f c (q x);
             abs x r
      | [p':Prop] p' => 
           h <- hash p' limit;
           match PositiveMap.find h c with
             | None => raise ProofNotFound
             | Some l => find p' l
           end
      end.
  
  Definition tauto P := 
    tauto' (PositiveMap.empty _) P.

End WithMap.

Example ex_wmap : True /\ (forall p:Prop, p -> p) := run (WithMap.tauto _).

Example ex_stress_0m 
  (p00 p01 p02 p03 p04 : Prop) :
  p00 -> p01 -> p02 -> p03 -> p04 ->
  p00 /\ p01 /\ p02 /\ p03 /\ p04.
Proof. 
  idtac "Map 5".
  Time refine (run (WithMap.tauto _)).
Qed.

Module WithHash.

  Definition ProofNotFound : Exception.
    exact exception.
  Qed.

  Definition ctx := HashTbl.t Prop (fun x=>x).

  Program
  Definition tauto' (c : ctx) :=
    mfix f (p : Prop) : M _ :=
    let f := f : forall p:Prop, M p in
    mmatch p as p' return M p' with
    | True => ret I 
    | [p1 p2] p1 /\ p2 =>
         r1 <- f p1 ;
         r2 <- f p2 ;
         ret (conj r1 r2)
    | [p1 p2]  p1 \/ p2 =>
         mtry 
           r1 <- f p1 ;
           ret (or_introl r1)
         with _ =>
           r2 <- f p2 ;
           ret (or_intror r2)
         end
    | [p1 p2 : Prop] p1 -> p2 =>
         nu (x:p1),
           HashTbl.add c p1 x;;
           mtry r <- f p2; HashTbl.remove c p1;; abs x r
           with [e] e => HashTbl.remove c p1;; raise e end
    | [A (q:A -> Prop)] (forall x:A, q x) =>
         nu (x:A),
           r <- f (q x);
           abs x r
    | [A (q:A -> Prop)] (exists x : A, q x) =>
         X <- evar A;
         mtry 
           r <- f (q X);
           ret (ex_intro q X r)
         with ProofNotFound =>
           r <- evar (q X);
           ret (ex_intro q X r)
         end
    | [x:Prop] x => 
      mtry HashTbl.find c x
      with HashTbl.NotFound => raise ProofNotFound
      end
    end.

  Definition tauto P := 
    c <- HashTbl.create Prop (fun x=>x);
    tauto' c P.


  Example should_fail : forall (P Q : Prop), (P -> Q) \/ (Q -> P).
  Proof.
    Fail refine (run (tauto _)).
    (* Should say "WithHash.ProofNotFound" *)
  Abort.

  Example should_fail : forall (P Q : Prop), (P -> P) /\ (P -> Q).
  Proof.
    Fail refine (run (tauto _)).
    (* Should say "WithHash.ProofNotFound" *)
  Abort.

  Example ex_first_order_2 (p q r : Prop) : 
    q -> p -> q.
  Proof. 
    apply (eval (tauto _)).
  Qed.

  Example ex_existential1 : exists P : Prop, P -> P.
  Proof.
    refine (run (tauto _)).
  Qed.

  Example ex_existential : forall x : nat, exists y, y <= x.
  Proof.
    refine (run (tauto _)).
    apply Le.le_0_n.
  Qed.

End WithHash.


(** * Performance test ***)

Example ex_stress_1 
  (p00 p01 p02 p03 p04 p05 p06 p07 p08 p09 : Prop) :
  p00 -> p01 -> p02 -> p03 -> p04 -> p05 -> p06 -> p07 -> p08 -> p09 ->
  p00 /\ p01 /\ p02 /\ p03 /\ p04 /\ p05 /\ p06 /\ p07 /\ p08 /\ p09.
Proof. 
  idtac "Hash 10".
  Time refine (run (WithHash.tauto _)).
Qed.

Example ex_stress_2
  (p00 p01 p02 p03 p04 p05 p06 p07 p08 p09 : Prop)
  (p10 p11 p12 p13 p14 p15 p16 p17 p18 p19 : Prop) :

  p00 -> p01 -> p02 -> p03 -> p04 -> p05 -> p06 -> p07 -> p08 -> p09 ->
  p10 -> p11 -> p12 -> p13 -> p14 -> p15 -> p16 -> p17 -> p18 -> p19 ->

  p00 /\ p01 /\ p02 /\ p03 /\ p04 /\ p05 /\ p06 /\ p07 /\ p08 /\ p09 /\
  p10 /\ p11 /\ p12 /\ p13 /\ p14 /\ p15 /\ p16 /\ p17 /\ p18 /\ p19.
Proof. 
  idtac "Hash 20".
  Time refine (run (WithHash.tauto _)).
Qed.

Example ex_stress_3
  (p00 p01 p02 p03 p04 p05 p06 p07 p08 p09 : Prop)
  (p10 p11 p12 p13 p14 p15 p16 p17 p18 p19 : Prop)
  (p20 p21 p22 p23 p24 p25 p26 p27 p28 p29 : Prop) :

  p00 -> p01 -> p02 -> p03 -> p04 -> p05 -> p06 -> p07 -> p08 -> p09 ->
  p10 -> p11 -> p12 -> p13 -> p14 -> p15 -> p16 -> p17 -> p18 -> p19 ->
  p20 -> p21 -> p22 -> p23 -> p24 -> p25 -> p26 -> p27 -> p28 -> p29 ->

  p00 /\ p01 /\ p02 /\ p03 /\ p04 /\ p05 /\ p06 /\ p07 /\ p08 /\ p09 /\
  p10 /\ p11 /\ p12 /\ p13 /\ p14 /\ p15 /\ p16 /\ p17 /\ p18 /\ p19 /\
  p20 /\ p21 /\ p22 /\ p23 /\ p24 /\ p25 /\ p26 /\ p27 /\ p28 /\ p29.
Proof. 
  idtac "Hash 30".
  Time refine (run (WithHash.tauto _)).
Qed.

Example ex_stress_4
  (p00 p01 p02 p03 p04 p05 p06 p07 p08 p09 : Prop)
  (p10 p11 p12 p13 p14 p15 p16 p17 p18 p19 : Prop)
  (p20 p21 p22 p23 p24 p25 p26 p27 p28 p29 : Prop)
  (p30 p31 p32 p33 p34 p35 p36 p37 p38 p39 : Prop) :

  p00 -> p01 -> p02 -> p03 -> p04 -> p05 -> p06 -> p07 -> p08 -> p09 ->
  p10 -> p11 -> p12 -> p13 -> p14 -> p15 -> p16 -> p17 -> p18 -> p19 ->
  p20 -> p21 -> p22 -> p23 -> p24 -> p25 -> p26 -> p27 -> p28 -> p29 ->
  p30 -> p31 -> p32 -> p33 -> p34 -> p35 -> p36 -> p37 -> p38 -> p39 ->

  p00 /\ p01 /\ p02 /\ p03 /\ p04 /\ p05 /\ p06 /\ p07 /\ p08 /\ p09 /\
  p10 /\ p11 /\ p12 /\ p13 /\ p14 /\ p15 /\ p16 /\ p17 /\ p18 /\ p19 /\
  p20 /\ p21 /\ p22 /\ p23 /\ p24 /\ p25 /\ p26 /\ p27 /\ p28 /\ p29 /\
  p30 /\ p31 /\ p32 /\ p33 /\ p34 /\ p35 /\ p36 /\ p37 /\ p38 /\ p39.
Proof. 
  idtac "Hash 40".
  Time refine (run (WithHash.tauto _)).
Qed.


Example ex_stress_5
  (p00 p01 p02 p03 p04 p05 p06 p07 p08 p09 : Prop)
  (p10 p11 p12 p13 p14 p15 p16 p17 p18 p19 : Prop)
  (p20 p21 p22 p23 p24 p25 p26 p27 p28 p29 : Prop)
  (p30 p31 p32 p33 p34 p35 p36 p37 p38 p39 : Prop) 
  (p40 p41 p42 p43 p44 p45 p46 p47 p48 p49 : Prop) :

  p00 -> p01 -> p02 -> p03 -> p04 -> p05 -> p06 -> p07 -> p08 -> p09 ->
  p10 -> p11 -> p12 -> p13 -> p14 -> p15 -> p16 -> p17 -> p18 -> p19 ->
  p20 -> p21 -> p22 -> p23 -> p24 -> p25 -> p26 -> p27 -> p28 -> p29 ->
  p30 -> p31 -> p32 -> p33 -> p34 -> p35 -> p36 -> p37 -> p38 -> p39 ->
  p40 -> p41 -> p42 -> p43 -> p44 -> p45 -> p46 -> p47 -> p48 -> p49 ->

  p00 /\ p01 /\ p02 /\ p03 /\ p04 /\ p05 /\ p06 /\ p07 /\ p08 /\ p09 /\
  p10 /\ p11 /\ p12 /\ p13 /\ p14 /\ p15 /\ p16 /\ p17 /\ p18 /\ p19 /\
  p20 /\ p21 /\ p22 /\ p23 /\ p24 /\ p25 /\ p26 /\ p27 /\ p28 /\ p29 /\
  p30 /\ p31 /\ p32 /\ p33 /\ p34 /\ p35 /\ p36 /\ p37 /\ p38 /\ p39 /\
  p40 /\ p41 /\ p42 /\ p43 /\ p44 /\ p45 /\ p46 /\ p47 /\ p48 /\ p49.
Proof. 
  idtac "Hash 50".
  Time refine (run (WithHash.tauto _)).
Qed.



Example ex_stress_6
  (p00 p01 p02 p03 p04 p05 p06 p07 p08 p09 : Prop)
  (p10 p11 p12 p13 p14 p15 p16 p17 p18 p19 : Prop)
  (p20 p21 p22 p23 p24 p25 p26 p27 p28 p29 : Prop)
  (p30 p31 p32 p33 p34 p35 p36 p37 p38 p39 : Prop) 
  (p40 p41 p42 p43 p44 p45 p46 p47 p48 p49 : Prop)
  (p50 p51 p52 p53 p54 p55 p56 p57 p58 p59 : Prop) :

  p00 -> p01 -> p02 -> p03 -> p04 -> p05 -> p06 -> p07 -> p08 -> p09 ->
  p10 -> p11 -> p12 -> p13 -> p14 -> p15 -> p16 -> p17 -> p18 -> p19 ->
  p20 -> p21 -> p22 -> p23 -> p24 -> p25 -> p26 -> p27 -> p28 -> p29 ->
  p30 -> p31 -> p32 -> p33 -> p34 -> p35 -> p36 -> p37 -> p38 -> p39 ->
  p40 -> p41 -> p42 -> p43 -> p44 -> p45 -> p46 -> p47 -> p48 -> p49 ->
  p50 -> p51 -> p52 -> p53 -> p54 -> p55 -> p56 -> p57 -> p58 -> p59 -> 

  p00 /\ p01 /\ p02 /\ p03 /\ p04 /\ p05 /\ p06 /\ p07 /\ p08 /\ p09 /\
  p10 /\ p11 /\ p12 /\ p13 /\ p14 /\ p15 /\ p16 /\ p17 /\ p18 /\ p19 /\
  p20 /\ p21 /\ p22 /\ p23 /\ p24 /\ p25 /\ p26 /\ p27 /\ p28 /\ p29 /\
  p30 /\ p31 /\ p32 /\ p33 /\ p34 /\ p35 /\ p36 /\ p37 /\ p38 /\ p39 /\
  p40 /\ p41 /\ p42 /\ p43 /\ p44 /\ p45 /\ p46 /\ p47 /\ p48 /\ p49 /\
  p50 /\ p51 /\ p52 /\ p53 /\ p54 /\ p55 /\ p56 /\ p57 /\ p58 /\ p59.
Proof. 
  idtac "Hash 60".
  Time refine (run (WithHash.tauto _)).
Qed.

Example ex_stress_7
  (p00 p01 p02 p03 p04 p05 p06 p07 p08 p09 : Prop)
  (p10 p11 p12 p13 p14 p15 p16 p17 p18 p19 : Prop)
  (p20 p21 p22 p23 p24 p25 p26 p27 p28 p29 : Prop)
  (p30 p31 p32 p33 p34 p35 p36 p37 p38 p39 : Prop) 
  (p40 p41 p42 p43 p44 p45 p46 p47 p48 p49 : Prop)
  (p50 p51 p52 p53 p54 p55 p56 p57 p58 p59 : Prop)
  (p60 p61 p62 p63 p64 p65 p66 p67 p68 p69 : Prop) :

  p00 -> p01 -> p02 -> p03 -> p04 -> p05 -> p06 -> p07 -> p08 -> p09 ->
  p10 -> p11 -> p12 -> p13 -> p14 -> p15 -> p16 -> p17 -> p18 -> p19 ->
  p20 -> p21 -> p22 -> p23 -> p24 -> p25 -> p26 -> p27 -> p28 -> p29 ->
  p30 -> p31 -> p32 -> p33 -> p34 -> p35 -> p36 -> p37 -> p38 -> p39 ->
  p40 -> p41 -> p42 -> p43 -> p44 -> p45 -> p46 -> p47 -> p48 -> p49 ->
  p50 -> p51 -> p52 -> p53 -> p54 -> p55 -> p56 -> p57 -> p58 -> p59 -> 
  p60 -> p61 -> p62 -> p63 -> p64 -> p65 -> p66 -> p67 -> p68 -> p69 -> 

  p00 /\ p01 /\ p02 /\ p03 /\ p04 /\ p05 /\ p06 /\ p07 /\ p08 /\ p09 /\
  p10 /\ p11 /\ p12 /\ p13 /\ p14 /\ p15 /\ p16 /\ p17 /\ p18 /\ p19 /\
  p20 /\ p21 /\ p22 /\ p23 /\ p24 /\ p25 /\ p26 /\ p27 /\ p28 /\ p29 /\
  p30 /\ p31 /\ p32 /\ p33 /\ p34 /\ p35 /\ p36 /\ p37 /\ p38 /\ p39 /\
  p40 /\ p41 /\ p42 /\ p43 /\ p44 /\ p45 /\ p46 /\ p47 /\ p48 /\ p49 /\
  p50 /\ p51 /\ p52 /\ p53 /\ p54 /\ p55 /\ p56 /\ p57 /\ p58 /\ p59 /\
  p60 /\ p61 /\ p62 /\ p63 /\ p64 /\ p65 /\ p66 /\ p67 /\ p68 /\ p69.
Proof. 
  idtac "Hash 70".
  Time refine (run (WithHash.tauto _)).
Qed.


Example ex_stress_8
  (p00 p01 p02 p03 p04 p05 p06 p07 p08 p09 : Prop)
  (p10 p11 p12 p13 p14 p15 p16 p17 p18 p19 : Prop)
  (p20 p21 p22 p23 p24 p25 p26 p27 p28 p29 : Prop)
  (p30 p31 p32 p33 p34 p35 p36 p37 p38 p39 : Prop) 
  (p40 p41 p42 p43 p44 p45 p46 p47 p48 p49 : Prop)
  (p50 p51 p52 p53 p54 p55 p56 p57 p58 p59 : Prop)
  (p60 p61 p62 p63 p64 p65 p66 p67 p68 p69 : Prop)
  (p70 p71 p72 p73 p74 p75 p76 p77 p78 p79 : Prop) :

  p00 -> p01 -> p02 -> p03 -> p04 -> p05 -> p06 -> p07 -> p08 -> p09 ->
  p10 -> p11 -> p12 -> p13 -> p14 -> p15 -> p16 -> p17 -> p18 -> p19 ->
  p20 -> p21 -> p22 -> p23 -> p24 -> p25 -> p26 -> p27 -> p28 -> p29 ->
  p30 -> p31 -> p32 -> p33 -> p34 -> p35 -> p36 -> p37 -> p38 -> p39 ->
  p40 -> p41 -> p42 -> p43 -> p44 -> p45 -> p46 -> p47 -> p48 -> p49 ->
  p50 -> p51 -> p52 -> p53 -> p54 -> p55 -> p56 -> p57 -> p58 -> p59 -> 
  p60 -> p61 -> p62 -> p63 -> p64 -> p65 -> p66 -> p67 -> p68 -> p69 -> 
  p70 -> p71 -> p72 -> p73 -> p74 -> p75 -> p76 -> p77 -> p78 -> p79 -> 

  p00 /\ p01 /\ p02 /\ p03 /\ p04 /\ p05 /\ p06 /\ p07 /\ p08 /\ p09 /\
  p10 /\ p11 /\ p12 /\ p13 /\ p14 /\ p15 /\ p16 /\ p17 /\ p18 /\ p19 /\
  p20 /\ p21 /\ p22 /\ p23 /\ p24 /\ p25 /\ p26 /\ p27 /\ p28 /\ p29 /\
  p30 /\ p31 /\ p32 /\ p33 /\ p34 /\ p35 /\ p36 /\ p37 /\ p38 /\ p39 /\
  p40 /\ p41 /\ p42 /\ p43 /\ p44 /\ p45 /\ p46 /\ p47 /\ p48 /\ p49 /\
  p50 /\ p51 /\ p52 /\ p53 /\ p54 /\ p55 /\ p56 /\ p57 /\ p58 /\ p59 /\
  p60 /\ p61 /\ p62 /\ p63 /\ p64 /\ p65 /\ p66 /\ p67 /\ p68 /\ p69 /\
  p70 /\ p71 /\ p72 /\ p73 /\ p74 /\ p75 /\ p76 /\ p77 /\ p78 /\ p79.
Proof. 
  idtac "Hash 80".
  Time refine (run (WithHash.tauto _)).
Qed.


Example ex_stress_9
  (p00 p01 p02 p03 p04 p05 p06 p07 p08 p09 : Prop)
  (p10 p11 p12 p13 p14 p15 p16 p17 p18 p19 : Prop)
  (p20 p21 p22 p23 p24 p25 p26 p27 p28 p29 : Prop)
  (p30 p31 p32 p33 p34 p35 p36 p37 p38 p39 : Prop) 
  (p40 p41 p42 p43 p44 p45 p46 p47 p48 p49 : Prop)
  (p50 p51 p52 p53 p54 p55 p56 p57 p58 p59 : Prop)
  (p60 p61 p62 p63 p64 p65 p66 p67 p68 p69 : Prop)
  (p70 p71 p72 p73 p74 p75 p76 p77 p78 p79 : Prop)
  (p80 p81 p82 p83 p84 p85 p86 p87 p88 p89 : Prop) :

  p00 -> p01 -> p02 -> p03 -> p04 -> p05 -> p06 -> p07 -> p08 -> p09 ->
  p10 -> p11 -> p12 -> p13 -> p14 -> p15 -> p16 -> p17 -> p18 -> p19 ->
  p20 -> p21 -> p22 -> p23 -> p24 -> p25 -> p26 -> p27 -> p28 -> p29 ->
  p30 -> p31 -> p32 -> p33 -> p34 -> p35 -> p36 -> p37 -> p38 -> p39 ->
  p40 -> p41 -> p42 -> p43 -> p44 -> p45 -> p46 -> p47 -> p48 -> p49 ->
  p50 -> p51 -> p52 -> p53 -> p54 -> p55 -> p56 -> p57 -> p58 -> p59 -> 
  p60 -> p61 -> p62 -> p63 -> p64 -> p65 -> p66 -> p67 -> p68 -> p69 -> 
  p70 -> p71 -> p72 -> p73 -> p74 -> p75 -> p76 -> p77 -> p78 -> p79 -> 
  p80 -> p81 -> p82 -> p83 -> p84 -> p85 -> p86 -> p87 -> p88 -> p89 -> 

  p00 /\ p01 /\ p02 /\ p03 /\ p04 /\ p05 /\ p06 /\ p07 /\ p08 /\ p09 /\
  p10 /\ p11 /\ p12 /\ p13 /\ p14 /\ p15 /\ p16 /\ p17 /\ p18 /\ p19 /\
  p20 /\ p21 /\ p22 /\ p23 /\ p24 /\ p25 /\ p26 /\ p27 /\ p28 /\ p29 /\
  p30 /\ p31 /\ p32 /\ p33 /\ p34 /\ p35 /\ p36 /\ p37 /\ p38 /\ p39 /\
  p40 /\ p41 /\ p42 /\ p43 /\ p44 /\ p45 /\ p46 /\ p47 /\ p48 /\ p49 /\
  p50 /\ p51 /\ p52 /\ p53 /\ p54 /\ p55 /\ p56 /\ p57 /\ p58 /\ p59 /\
  p60 /\ p61 /\ p62 /\ p63 /\ p64 /\ p65 /\ p66 /\ p67 /\ p68 /\ p69 /\
  p70 /\ p71 /\ p72 /\ p73 /\ p74 /\ p75 /\ p76 /\ p77 /\ p78 /\ p79 /\
  p80 /\ p81 /\ p82 /\ p83 /\ p84 /\ p85 /\ p86 /\ p87 /\ p88 /\ p89.
Proof. 
  idtac "Hash 90".
  Time refine (run (WithHash.tauto _)).
Qed.


Example ex_stress_10
  (p00 p01 p02 p03 p04 p05 p06 p07 p08 p09 : Prop)
  (p10 p11 p12 p13 p14 p15 p16 p17 p18 p19 : Prop)
  (p20 p21 p22 p23 p24 p25 p26 p27 p28 p29 : Prop)
  (p30 p31 p32 p33 p34 p35 p36 p37 p38 p39 : Prop) 
  (p40 p41 p42 p43 p44 p45 p46 p47 p48 p49 : Prop)
  (p50 p51 p52 p53 p54 p55 p56 p57 p58 p59 : Prop)
  (p60 p61 p62 p63 p64 p65 p66 p67 p68 p69 : Prop)
  (p70 p71 p72 p73 p74 p75 p76 p77 p78 p79 : Prop)
  (p80 p81 p82 p83 p84 p85 p86 p87 p88 p89 : Prop)
  (p90 p91 p92 p93 p94 p95 p96 p97 p98 p99 : Prop) :

  p00 -> p01 -> p02 -> p03 -> p04 -> p05 -> p06 -> p07 -> p08 -> p09 ->
  p10 -> p11 -> p12 -> p13 -> p14 -> p15 -> p16 -> p17 -> p18 -> p19 ->
  p20 -> p21 -> p22 -> p23 -> p24 -> p25 -> p26 -> p27 -> p28 -> p29 ->
  p30 -> p31 -> p32 -> p33 -> p34 -> p35 -> p36 -> p37 -> p38 -> p39 ->
  p40 -> p41 -> p42 -> p43 -> p44 -> p45 -> p46 -> p47 -> p48 -> p49 ->
  p50 -> p51 -> p52 -> p53 -> p54 -> p55 -> p56 -> p57 -> p58 -> p59 -> 
  p60 -> p61 -> p62 -> p63 -> p64 -> p65 -> p66 -> p67 -> p68 -> p69 -> 
  p70 -> p71 -> p72 -> p73 -> p74 -> p75 -> p76 -> p77 -> p78 -> p79 -> 
  p80 -> p81 -> p82 -> p83 -> p84 -> p85 -> p86 -> p87 -> p88 -> p89 -> 
  p90 -> p91 -> p92 -> p93 -> p94 -> p95 -> p96 -> p97 -> p98 -> p99 -> 

  p00 /\ p01 /\ p02 /\ p03 /\ p04 /\ p05 /\ p06 /\ p07 /\ p08 /\ p09 /\
  p10 /\ p11 /\ p12 /\ p13 /\ p14 /\ p15 /\ p16 /\ p17 /\ p18 /\ p19 /\
  p20 /\ p21 /\ p22 /\ p23 /\ p24 /\ p25 /\ p26 /\ p27 /\ p28 /\ p29 /\
  p30 /\ p31 /\ p32 /\ p33 /\ p34 /\ p35 /\ p36 /\ p37 /\ p38 /\ p39 /\
  p40 /\ p41 /\ p42 /\ p43 /\ p44 /\ p45 /\ p46 /\ p47 /\ p48 /\ p49 /\
  p50 /\ p51 /\ p52 /\ p53 /\ p54 /\ p55 /\ p56 /\ p57 /\ p58 /\ p59 /\
  p60 /\ p61 /\ p62 /\ p63 /\ p64 /\ p65 /\ p66 /\ p67 /\ p68 /\ p69 /\
  p70 /\ p71 /\ p72 /\ p73 /\ p74 /\ p75 /\ p76 /\ p77 /\ p78 /\ p79 /\
  p80 /\ p81 /\ p82 /\ p83 /\ p84 /\ p85 /\ p86 /\ p87 /\ p88 /\ p89 /\
  p90 /\ p91 /\ p92 /\ p93 /\ p94 /\ p95 /\ p96 /\ p97 /\ p98 /\ p99.
Proof. 
  idtac "Hash 100".
  Time refine (run (WithHash.tauto _)).
Qed.






Example ex_stress_1m 
  (p00 p01 p02 p03 p04 p05 p06 p07 p08 p09 : Prop) :
  p00 -> p01 -> p02 -> p03 -> p04 -> p05 -> p06 -> p07 -> p08 -> p09 ->
  p00 /\ p01 /\ p02 /\ p03 /\ p04 /\ p05 /\ p06 /\ p07 /\ p08 /\ p09.
Proof. 
  idtac "Map 10".
  Time refine (run (WithMap.tauto _)).
Qed.

Example ex_stress_2m
  (p00 p01 p02 p03 p04 p05 p06 p07 p08 p09 : Prop)
  (p10 p11 p12 p13 p14 p15 p16 p17 p18 p19 : Prop) :

  p00 -> p01 -> p02 -> p03 -> p04 -> p05 -> p06 -> p07 -> p08 -> p09 ->
  p10 -> p11 -> p12 -> p13 -> p14 -> p15 -> p16 -> p17 -> p18 -> p19 ->

  p00 /\ p01 /\ p02 /\ p03 /\ p04 /\ p05 /\ p06 /\ p07 /\ p08 /\ p09 /\
  p10 /\ p11 /\ p12 /\ p13 /\ p14 /\ p15 /\ p16 /\ p17 /\ p18 /\ p19.
Proof. 
  idtac "Map 20".
  Time refine (run (WithMap.tauto _)).
Qed.

Example ex_stress_3m
  (p00 p01 p02 p03 p04 p05 p06 p07 p08 p09 : Prop)
  (p10 p11 p12 p13 p14 p15 p16 p17 p18 p19 : Prop)
  (p20 p21 p22 p23 p24 p25 p26 p27 p28 p29 : Prop) :

  p00 -> p01 -> p02 -> p03 -> p04 -> p05 -> p06 -> p07 -> p08 -> p09 ->
  p10 -> p11 -> p12 -> p13 -> p14 -> p15 -> p16 -> p17 -> p18 -> p19 ->
  p20 -> p21 -> p22 -> p23 -> p24 -> p25 -> p26 -> p27 -> p28 -> p29 ->

  p00 /\ p01 /\ p02 /\ p03 /\ p04 /\ p05 /\ p06 /\ p07 /\ p08 /\ p09 /\
  p10 /\ p11 /\ p12 /\ p13 /\ p14 /\ p15 /\ p16 /\ p17 /\ p18 /\ p19 /\
  p20 /\ p21 /\ p22 /\ p23 /\ p24 /\ p25 /\ p26 /\ p27 /\ p28 /\ p29.
Proof. 
  idtac "Map 30".
  Time refine (run (WithMap.tauto _)).
Qed.

Example ex_stress_4m
  (p00 p01 p02 p03 p04 p05 p06 p07 p08 p09 : Prop)
  (p10 p11 p12 p13 p14 p15 p16 p17 p18 p19 : Prop)
  (p20 p21 p22 p23 p24 p25 p26 p27 p28 p29 : Prop)
  (p30 p31 p32 p33 p34 p35 p36 p37 p38 p39 : Prop) :

  p00 -> p01 -> p02 -> p03 -> p04 -> p05 -> p06 -> p07 -> p08 -> p09 ->
  p10 -> p11 -> p12 -> p13 -> p14 -> p15 -> p16 -> p17 -> p18 -> p19 ->
  p20 -> p21 -> p22 -> p23 -> p24 -> p25 -> p26 -> p27 -> p28 -> p29 ->
  p30 -> p31 -> p32 -> p33 -> p34 -> p35 -> p36 -> p37 -> p38 -> p39 ->

  p00 /\ p01 /\ p02 /\ p03 /\ p04 /\ p05 /\ p06 /\ p07 /\ p08 /\ p09 /\
  p10 /\ p11 /\ p12 /\ p13 /\ p14 /\ p15 /\ p16 /\ p17 /\ p18 /\ p19 /\
  p20 /\ p21 /\ p22 /\ p23 /\ p24 /\ p25 /\ p26 /\ p27 /\ p28 /\ p29 /\
  p30 /\ p31 /\ p32 /\ p33 /\ p34 /\ p35 /\ p36 /\ p37 /\ p38 /\ p39.
Proof. 
  idtac "Map 40".
  Time refine (run (WithMap.tauto _)).
Qed.


Example ex_stress_5m
  (p00 p01 p02 p03 p04 p05 p06 p07 p08 p09 : Prop)
  (p10 p11 p12 p13 p14 p15 p16 p17 p18 p19 : Prop)
  (p20 p21 p22 p23 p24 p25 p26 p27 p28 p29 : Prop)
  (p30 p31 p32 p33 p34 p35 p36 p37 p38 p39 : Prop) 
  (p40 p41 p42 p43 p44 p45 p46 p47 p48 p49 : Prop) :

  p00 -> p01 -> p02 -> p03 -> p04 -> p05 -> p06 -> p07 -> p08 -> p09 ->
  p10 -> p11 -> p12 -> p13 -> p14 -> p15 -> p16 -> p17 -> p18 -> p19 ->
  p20 -> p21 -> p22 -> p23 -> p24 -> p25 -> p26 -> p27 -> p28 -> p29 ->
  p30 -> p31 -> p32 -> p33 -> p34 -> p35 -> p36 -> p37 -> p38 -> p39 ->
  p40 -> p41 -> p42 -> p43 -> p44 -> p45 -> p46 -> p47 -> p48 -> p49 ->

  p00 /\ p01 /\ p02 /\ p03 /\ p04 /\ p05 /\ p06 /\ p07 /\ p08 /\ p09 /\
  p10 /\ p11 /\ p12 /\ p13 /\ p14 /\ p15 /\ p16 /\ p17 /\ p18 /\ p19 /\
  p20 /\ p21 /\ p22 /\ p23 /\ p24 /\ p25 /\ p26 /\ p27 /\ p28 /\ p29 /\
  p30 /\ p31 /\ p32 /\ p33 /\ p34 /\ p35 /\ p36 /\ p37 /\ p38 /\ p39 /\
  p40 /\ p41 /\ p42 /\ p43 /\ p44 /\ p45 /\ p46 /\ p47 /\ p48 /\ p49.
Proof. 
  idtac "Map 50".
  Time refine (run (WithMap.tauto _)).
Qed.



Example ex_stress_6m
  (p00 p01 p02 p03 p04 p05 p06 p07 p08 p09 : Prop)
  (p10 p11 p12 p13 p14 p15 p16 p17 p18 p19 : Prop)
  (p20 p21 p22 p23 p24 p25 p26 p27 p28 p29 : Prop)
  (p30 p31 p32 p33 p34 p35 p36 p37 p38 p39 : Prop) 
  (p40 p41 p42 p43 p44 p45 p46 p47 p48 p49 : Prop)
  (p50 p51 p52 p53 p54 p55 p56 p57 p58 p59 : Prop) :

  p00 -> p01 -> p02 -> p03 -> p04 -> p05 -> p06 -> p07 -> p08 -> p09 ->
  p10 -> p11 -> p12 -> p13 -> p14 -> p15 -> p16 -> p17 -> p18 -> p19 ->
  p20 -> p21 -> p22 -> p23 -> p24 -> p25 -> p26 -> p27 -> p28 -> p29 ->
  p30 -> p31 -> p32 -> p33 -> p34 -> p35 -> p36 -> p37 -> p38 -> p39 ->
  p40 -> p41 -> p42 -> p43 -> p44 -> p45 -> p46 -> p47 -> p48 -> p49 ->
  p50 -> p51 -> p52 -> p53 -> p54 -> p55 -> p56 -> p57 -> p58 -> p59 -> 

  p00 /\ p01 /\ p02 /\ p03 /\ p04 /\ p05 /\ p06 /\ p07 /\ p08 /\ p09 /\
  p10 /\ p11 /\ p12 /\ p13 /\ p14 /\ p15 /\ p16 /\ p17 /\ p18 /\ p19 /\
  p20 /\ p21 /\ p22 /\ p23 /\ p24 /\ p25 /\ p26 /\ p27 /\ p28 /\ p29 /\
  p30 /\ p31 /\ p32 /\ p33 /\ p34 /\ p35 /\ p36 /\ p37 /\ p38 /\ p39 /\
  p40 /\ p41 /\ p42 /\ p43 /\ p44 /\ p45 /\ p46 /\ p47 /\ p48 /\ p49 /\
  p50 /\ p51 /\ p52 /\ p53 /\ p54 /\ p55 /\ p56 /\ p57 /\ p58 /\ p59.
Proof. 
  idtac "Map 60".
  Time refine (run (WithMap.tauto _)).
Qed.

Example ex_stress_7m
  (p00 p01 p02 p03 p04 p05 p06 p07 p08 p09 : Prop)
  (p10 p11 p12 p13 p14 p15 p16 p17 p18 p19 : Prop)
  (p20 p21 p22 p23 p24 p25 p26 p27 p28 p29 : Prop)
  (p30 p31 p32 p33 p34 p35 p36 p37 p38 p39 : Prop) 
  (p40 p41 p42 p43 p44 p45 p46 p47 p48 p49 : Prop)
  (p50 p51 p52 p53 p54 p55 p56 p57 p58 p59 : Prop)
  (p60 p61 p62 p63 p64 p65 p66 p67 p68 p69 : Prop) :

  p00 -> p01 -> p02 -> p03 -> p04 -> p05 -> p06 -> p07 -> p08 -> p09 ->
  p10 -> p11 -> p12 -> p13 -> p14 -> p15 -> p16 -> p17 -> p18 -> p19 ->
  p20 -> p21 -> p22 -> p23 -> p24 -> p25 -> p26 -> p27 -> p28 -> p29 ->
  p30 -> p31 -> p32 -> p33 -> p34 -> p35 -> p36 -> p37 -> p38 -> p39 ->
  p40 -> p41 -> p42 -> p43 -> p44 -> p45 -> p46 -> p47 -> p48 -> p49 ->
  p50 -> p51 -> p52 -> p53 -> p54 -> p55 -> p56 -> p57 -> p58 -> p59 -> 
  p60 -> p61 -> p62 -> p63 -> p64 -> p65 -> p66 -> p67 -> p68 -> p69 -> 

  p00 /\ p01 /\ p02 /\ p03 /\ p04 /\ p05 /\ p06 /\ p07 /\ p08 /\ p09 /\
  p10 /\ p11 /\ p12 /\ p13 /\ p14 /\ p15 /\ p16 /\ p17 /\ p18 /\ p19 /\
  p20 /\ p21 /\ p22 /\ p23 /\ p24 /\ p25 /\ p26 /\ p27 /\ p28 /\ p29 /\
  p30 /\ p31 /\ p32 /\ p33 /\ p34 /\ p35 /\ p36 /\ p37 /\ p38 /\ p39 /\
  p40 /\ p41 /\ p42 /\ p43 /\ p44 /\ p45 /\ p46 /\ p47 /\ p48 /\ p49 /\
  p50 /\ p51 /\ p52 /\ p53 /\ p54 /\ p55 /\ p56 /\ p57 /\ p58 /\ p59 /\
  p60 /\ p61 /\ p62 /\ p63 /\ p64 /\ p65 /\ p66 /\ p67 /\ p68 /\ p69.
Proof. 
  idtac "Map 70".
  Time refine (run (WithMap.tauto _)).
Qed.


Example ex_stress_8m
  (p00 p01 p02 p03 p04 p05 p06 p07 p08 p09 : Prop)
  (p10 p11 p12 p13 p14 p15 p16 p17 p18 p19 : Prop)
  (p20 p21 p22 p23 p24 p25 p26 p27 p28 p29 : Prop)
  (p30 p31 p32 p33 p34 p35 p36 p37 p38 p39 : Prop) 
  (p40 p41 p42 p43 p44 p45 p46 p47 p48 p49 : Prop)
  (p50 p51 p52 p53 p54 p55 p56 p57 p58 p59 : Prop)
  (p60 p61 p62 p63 p64 p65 p66 p67 p68 p69 : Prop)
  (p70 p71 p72 p73 p74 p75 p76 p77 p78 p79 : Prop) :

  p00 -> p01 -> p02 -> p03 -> p04 -> p05 -> p06 -> p07 -> p08 -> p09 ->
  p10 -> p11 -> p12 -> p13 -> p14 -> p15 -> p16 -> p17 -> p18 -> p19 ->
  p20 -> p21 -> p22 -> p23 -> p24 -> p25 -> p26 -> p27 -> p28 -> p29 ->
  p30 -> p31 -> p32 -> p33 -> p34 -> p35 -> p36 -> p37 -> p38 -> p39 ->
  p40 -> p41 -> p42 -> p43 -> p44 -> p45 -> p46 -> p47 -> p48 -> p49 ->
  p50 -> p51 -> p52 -> p53 -> p54 -> p55 -> p56 -> p57 -> p58 -> p59 -> 
  p60 -> p61 -> p62 -> p63 -> p64 -> p65 -> p66 -> p67 -> p68 -> p69 -> 
  p70 -> p71 -> p72 -> p73 -> p74 -> p75 -> p76 -> p77 -> p78 -> p79 -> 

  p00 /\ p01 /\ p02 /\ p03 /\ p04 /\ p05 /\ p06 /\ p07 /\ p08 /\ p09 /\
  p10 /\ p11 /\ p12 /\ p13 /\ p14 /\ p15 /\ p16 /\ p17 /\ p18 /\ p19 /\
  p20 /\ p21 /\ p22 /\ p23 /\ p24 /\ p25 /\ p26 /\ p27 /\ p28 /\ p29 /\
  p30 /\ p31 /\ p32 /\ p33 /\ p34 /\ p35 /\ p36 /\ p37 /\ p38 /\ p39 /\
  p40 /\ p41 /\ p42 /\ p43 /\ p44 /\ p45 /\ p46 /\ p47 /\ p48 /\ p49 /\
  p50 /\ p51 /\ p52 /\ p53 /\ p54 /\ p55 /\ p56 /\ p57 /\ p58 /\ p59 /\
  p60 /\ p61 /\ p62 /\ p63 /\ p64 /\ p65 /\ p66 /\ p67 /\ p68 /\ p69 /\
  p70 /\ p71 /\ p72 /\ p73 /\ p74 /\ p75 /\ p76 /\ p77 /\ p78 /\ p79.
Proof. 
  idtac "Map 80".
  Time refine (run (WithMap.tauto _)).
Qed.


Example ex_stress_9m
  (p00 p01 p02 p03 p04 p05 p06 p07 p08 p09 : Prop)
  (p10 p11 p12 p13 p14 p15 p16 p17 p18 p19 : Prop)
  (p20 p21 p22 p23 p24 p25 p26 p27 p28 p29 : Prop)
  (p30 p31 p32 p33 p34 p35 p36 p37 p38 p39 : Prop) 
  (p40 p41 p42 p43 p44 p45 p46 p47 p48 p49 : Prop)
  (p50 p51 p52 p53 p54 p55 p56 p57 p58 p59 : Prop)
  (p60 p61 p62 p63 p64 p65 p66 p67 p68 p69 : Prop)
  (p70 p71 p72 p73 p74 p75 p76 p77 p78 p79 : Prop)
  (p80 p81 p82 p83 p84 p85 p86 p87 p88 p89 : Prop) :

  p00 -> p01 -> p02 -> p03 -> p04 -> p05 -> p06 -> p07 -> p08 -> p09 ->
  p10 -> p11 -> p12 -> p13 -> p14 -> p15 -> p16 -> p17 -> p18 -> p19 ->
  p20 -> p21 -> p22 -> p23 -> p24 -> p25 -> p26 -> p27 -> p28 -> p29 ->
  p30 -> p31 -> p32 -> p33 -> p34 -> p35 -> p36 -> p37 -> p38 -> p39 ->
  p40 -> p41 -> p42 -> p43 -> p44 -> p45 -> p46 -> p47 -> p48 -> p49 ->
  p50 -> p51 -> p52 -> p53 -> p54 -> p55 -> p56 -> p57 -> p58 -> p59 -> 
  p60 -> p61 -> p62 -> p63 -> p64 -> p65 -> p66 -> p67 -> p68 -> p69 -> 
  p70 -> p71 -> p72 -> p73 -> p74 -> p75 -> p76 -> p77 -> p78 -> p79 -> 
  p80 -> p81 -> p82 -> p83 -> p84 -> p85 -> p86 -> p87 -> p88 -> p89 -> 

  p00 /\ p01 /\ p02 /\ p03 /\ p04 /\ p05 /\ p06 /\ p07 /\ p08 /\ p09 /\
  p10 /\ p11 /\ p12 /\ p13 /\ p14 /\ p15 /\ p16 /\ p17 /\ p18 /\ p19 /\
  p20 /\ p21 /\ p22 /\ p23 /\ p24 /\ p25 /\ p26 /\ p27 /\ p28 /\ p29 /\
  p30 /\ p31 /\ p32 /\ p33 /\ p34 /\ p35 /\ p36 /\ p37 /\ p38 /\ p39 /\
  p40 /\ p41 /\ p42 /\ p43 /\ p44 /\ p45 /\ p46 /\ p47 /\ p48 /\ p49 /\
  p50 /\ p51 /\ p52 /\ p53 /\ p54 /\ p55 /\ p56 /\ p57 /\ p58 /\ p59 /\
  p60 /\ p61 /\ p62 /\ p63 /\ p64 /\ p65 /\ p66 /\ p67 /\ p68 /\ p69 /\
  p70 /\ p71 /\ p72 /\ p73 /\ p74 /\ p75 /\ p76 /\ p77 /\ p78 /\ p79 /\
  p80 /\ p81 /\ p82 /\ p83 /\ p84 /\ p85 /\ p86 /\ p87 /\ p88 /\ p89.
Proof. 
  idtac "Map 90".
  Time refine (run (WithMap.tauto _)).
Qed.


Example ex_stress_10m
  (p00 p01 p02 p03 p04 p05 p06 p07 p08 p09 : Prop)
  (p10 p11 p12 p13 p14 p15 p16 p17 p18 p19 : Prop)
  (p20 p21 p22 p23 p24 p25 p26 p27 p28 p29 : Prop)
  (p30 p31 p32 p33 p34 p35 p36 p37 p38 p39 : Prop) 
  (p40 p41 p42 p43 p44 p45 p46 p47 p48 p49 : Prop)
  (p50 p51 p52 p53 p54 p55 p56 p57 p58 p59 : Prop)
  (p60 p61 p62 p63 p64 p65 p66 p67 p68 p69 : Prop)
  (p70 p71 p72 p73 p74 p75 p76 p77 p78 p79 : Prop)
  (p80 p81 p82 p83 p84 p85 p86 p87 p88 p89 : Prop)
  (p90 p91 p92 p93 p94 p95 p96 p97 p98 p99 : Prop) :

  p00 -> p01 -> p02 -> p03 -> p04 -> p05 -> p06 -> p07 -> p08 -> p09 ->
  p10 -> p11 -> p12 -> p13 -> p14 -> p15 -> p16 -> p17 -> p18 -> p19 ->
  p20 -> p21 -> p22 -> p23 -> p24 -> p25 -> p26 -> p27 -> p28 -> p29 ->
  p30 -> p31 -> p32 -> p33 -> p34 -> p35 -> p36 -> p37 -> p38 -> p39 ->
  p40 -> p41 -> p42 -> p43 -> p44 -> p45 -> p46 -> p47 -> p48 -> p49 ->
  p50 -> p51 -> p52 -> p53 -> p54 -> p55 -> p56 -> p57 -> p58 -> p59 -> 
  p60 -> p61 -> p62 -> p63 -> p64 -> p65 -> p66 -> p67 -> p68 -> p69 -> 
  p70 -> p71 -> p72 -> p73 -> p74 -> p75 -> p76 -> p77 -> p78 -> p79 -> 
  p80 -> p81 -> p82 -> p83 -> p84 -> p85 -> p86 -> p87 -> p88 -> p89 -> 
  p90 -> p91 -> p92 -> p93 -> p94 -> p95 -> p96 -> p97 -> p98 -> p99 -> 

  p00 /\ p01 /\ p02 /\ p03 /\ p04 /\ p05 /\ p06 /\ p07 /\ p08 /\ p09 /\
  p10 /\ p11 /\ p12 /\ p13 /\ p14 /\ p15 /\ p16 /\ p17 /\ p18 /\ p19 /\
  p20 /\ p21 /\ p22 /\ p23 /\ p24 /\ p25 /\ p26 /\ p27 /\ p28 /\ p29 /\
  p30 /\ p31 /\ p32 /\ p33 /\ p34 /\ p35 /\ p36 /\ p37 /\ p38 /\ p39 /\
  p40 /\ p41 /\ p42 /\ p43 /\ p44 /\ p45 /\ p46 /\ p47 /\ p48 /\ p49 /\
  p50 /\ p51 /\ p52 /\ p53 /\ p54 /\ p55 /\ p56 /\ p57 /\ p58 /\ p59 /\
  p60 /\ p61 /\ p62 /\ p63 /\ p64 /\ p65 /\ p66 /\ p67 /\ p68 /\ p69 /\
  p70 /\ p71 /\ p72 /\ p73 /\ p74 /\ p75 /\ p76 /\ p77 /\ p78 /\ p79 /\
  p80 /\ p81 /\ p82 /\ p83 /\ p84 /\ p85 /\ p86 /\ p87 /\ p88 /\ p89 /\
  p90 /\ p91 /\ p92 /\ p93 /\ p94 /\ p95 /\ p96 /\ p97 /\ p98 /\ p99.
Proof. 
  idtac "Map 100".
  Time refine (run (WithMap.tauto _)).
Qed.







Example ex_stress_l1 
  (p00 p01 p02 p03 p04 p05 p06 p07 p08 p09 : Prop) :
  p00 -> p01 -> p02 -> p03 -> p04 -> p05 -> p06 -> p07 -> p08 -> p09 ->
  p00 /\ p01 /\ p02 /\ p03 /\ p04 /\ p05 /\ p06 /\ p07 /\ p08 /\ p09.
Proof. 
  idtac "List 10".
  Time refine (run (WithList.tauto _)).
Qed.

Example ex_stress_l2
  (p00 p01 p02 p03 p04 p05 p06 p07 p08 p09 : Prop)
  (p10 p11 p12 p13 p14 p15 p16 p17 p18 p19 : Prop) :

  p00 -> p01 -> p02 -> p03 -> p04 -> p05 -> p06 -> p07 -> p08 -> p09 ->
  p10 -> p11 -> p12 -> p13 -> p14 -> p15 -> p16 -> p17 -> p18 -> p19 ->

  p00 /\ p01 /\ p02 /\ p03 /\ p04 /\ p05 /\ p06 /\ p07 /\ p08 /\ p09 /\
  p10 /\ p11 /\ p12 /\ p13 /\ p14 /\ p15 /\ p16 /\ p17 /\ p18 /\ p19.
Proof. 
  idtac "List 20".
  Time refine (run (WithList.tauto _)).
Qed.

Example ex_stress_l3
  (p00 p01 p02 p03 p04 p05 p06 p07 p08 p09 : Prop)
  (p10 p11 p12 p13 p14 p15 p16 p17 p18 p19 : Prop)
  (p20 p21 p22 p23 p24 p25 p26 p27 p28 p29 : Prop) :

  p00 -> p01 -> p02 -> p03 -> p04 -> p05 -> p06 -> p07 -> p08 -> p09 ->
  p10 -> p11 -> p12 -> p13 -> p14 -> p15 -> p16 -> p17 -> p18 -> p19 ->
  p20 -> p21 -> p22 -> p23 -> p24 -> p25 -> p26 -> p27 -> p28 -> p29 ->

  p00 /\ p01 /\ p02 /\ p03 /\ p04 /\ p05 /\ p06 /\ p07 /\ p08 /\ p09 /\
  p10 /\ p11 /\ p12 /\ p13 /\ p14 /\ p15 /\ p16 /\ p17 /\ p18 /\ p19 /\
  p20 /\ p21 /\ p22 /\ p23 /\ p24 /\ p25 /\ p26 /\ p27 /\ p28 /\ p29.
Proof. 
  idtac "List 30".
  Time refine (run (WithList.tauto _)).
Qed.

Example ex_stress_l4
  (p00 p01 p02 p03 p04 p05 p06 p07 p08 p09 : Prop)
  (p10 p11 p12 p13 p14 p15 p16 p17 p18 p19 : Prop)
  (p20 p21 p22 p23 p24 p25 p26 p27 p28 p29 : Prop)
  (p30 p31 p32 p33 p34 p35 p36 p37 p38 p39 : Prop) :

  p00 -> p01 -> p02 -> p03 -> p04 -> p05 -> p06 -> p07 -> p08 -> p09 ->
  p10 -> p11 -> p12 -> p13 -> p14 -> p15 -> p16 -> p17 -> p18 -> p19 ->
  p20 -> p21 -> p22 -> p23 -> p24 -> p25 -> p26 -> p27 -> p28 -> p29 ->
  p30 -> p31 -> p32 -> p33 -> p34 -> p35 -> p36 -> p37 -> p38 -> p39 ->

  p00 /\ p01 /\ p02 /\ p03 /\ p04 /\ p05 /\ p06 /\ p07 /\ p08 /\ p09 /\
  p10 /\ p11 /\ p12 /\ p13 /\ p14 /\ p15 /\ p16 /\ p17 /\ p18 /\ p19 /\
  p20 /\ p21 /\ p22 /\ p23 /\ p24 /\ p25 /\ p26 /\ p27 /\ p28 /\ p29 /\
  p30 /\ p31 /\ p32 /\ p33 /\ p34 /\ p35 /\ p36 /\ p37 /\ p38 /\ p39.
Proof. 
  idtac "List 40".
  Time refine (run (WithList.tauto _)).
Qed.



Example ex_stress_l5
  (p00 p01 p02 p03 p04 p05 p06 p07 p08 p09 : Prop)
  (p10 p11 p12 p13 p14 p15 p16 p17 p18 p19 : Prop)
  (p20 p21 p22 p23 p24 p25 p26 p27 p28 p29 : Prop)
  (p30 p31 p32 p33 p34 p35 p36 p37 p38 p39 : Prop) 
  (p40 p41 p42 p43 p44 p45 p46 p47 p48 p49 : Prop) :

  p00 -> p01 -> p02 -> p03 -> p04 -> p05 -> p06 -> p07 -> p08 -> p09 ->
  p10 -> p11 -> p12 -> p13 -> p14 -> p15 -> p16 -> p17 -> p18 -> p19 ->
  p20 -> p21 -> p22 -> p23 -> p24 -> p25 -> p26 -> p27 -> p28 -> p29 ->
  p30 -> p31 -> p32 -> p33 -> p34 -> p35 -> p36 -> p37 -> p38 -> p39 ->
  p40 -> p41 -> p42 -> p43 -> p44 -> p45 -> p46 -> p47 -> p48 -> p49 ->

  p00 /\ p01 /\ p02 /\ p03 /\ p04 /\ p05 /\ p06 /\ p07 /\ p08 /\ p09 /\
  p10 /\ p11 /\ p12 /\ p13 /\ p14 /\ p15 /\ p16 /\ p17 /\ p18 /\ p19 /\
  p20 /\ p21 /\ p22 /\ p23 /\ p24 /\ p25 /\ p26 /\ p27 /\ p28 /\ p29 /\
  p30 /\ p31 /\ p32 /\ p33 /\ p34 /\ p35 /\ p36 /\ p37 /\ p38 /\ p39 /\
  p40 /\ p41 /\ p42 /\ p43 /\ p44 /\ p45 /\ p46 /\ p47 /\ p48 /\ p49.
Proof. 
  idtac "List 50".
  Time refine (run (WithList.tauto _)).
Qed.

Example ex_stress_l6
  (p00 p01 p02 p03 p04 p05 p06 p07 p08 p09 : Prop)
  (p10 p11 p12 p13 p14 p15 p16 p17 p18 p19 : Prop)
  (p20 p21 p22 p23 p24 p25 p26 p27 p28 p29 : Prop)
  (p30 p31 p32 p33 p34 p35 p36 p37 p38 p39 : Prop) 
  (p40 p41 p42 p43 p44 p45 p46 p47 p48 p49 : Prop)
  (p50 p51 p52 p53 p54 p55 p56 p57 p58 p59 : Prop) :

  p00 -> p01 -> p02 -> p03 -> p04 -> p05 -> p06 -> p07 -> p08 -> p09 ->
  p10 -> p11 -> p12 -> p13 -> p14 -> p15 -> p16 -> p17 -> p18 -> p19 ->
  p20 -> p21 -> p22 -> p23 -> p24 -> p25 -> p26 -> p27 -> p28 -> p29 ->
  p30 -> p31 -> p32 -> p33 -> p34 -> p35 -> p36 -> p37 -> p38 -> p39 ->
  p40 -> p41 -> p42 -> p43 -> p44 -> p45 -> p46 -> p47 -> p48 -> p49 ->
  p50 -> p51 -> p52 -> p53 -> p54 -> p55 -> p56 -> p57 -> p58 -> p59 -> 

  p00 /\ p01 /\ p02 /\ p03 /\ p04 /\ p05 /\ p06 /\ p07 /\ p08 /\ p09 /\
  p10 /\ p11 /\ p12 /\ p13 /\ p14 /\ p15 /\ p16 /\ p17 /\ p18 /\ p19 /\
  p20 /\ p21 /\ p22 /\ p23 /\ p24 /\ p25 /\ p26 /\ p27 /\ p28 /\ p29 /\
  p30 /\ p31 /\ p32 /\ p33 /\ p34 /\ p35 /\ p36 /\ p37 /\ p38 /\ p39 /\
  p40 /\ p41 /\ p42 /\ p43 /\ p44 /\ p45 /\ p46 /\ p47 /\ p48 /\ p49 /\
  p50 /\ p51 /\ p52 /\ p53 /\ p54 /\ p55 /\ p56 /\ p57 /\ p58 /\ p59.
Proof. 
  idtac "List 60".
  Time refine (run (WithList.tauto _)).
Qed.


Example ex_stress_l7
  (p00 p01 p02 p03 p04 p05 p06 p07 p08 p09 : Prop)
  (p10 p11 p12 p13 p14 p15 p16 p17 p18 p19 : Prop)
  (p20 p21 p22 p23 p24 p25 p26 p27 p28 p29 : Prop)
  (p30 p31 p32 p33 p34 p35 p36 p37 p38 p39 : Prop) 
  (p40 p41 p42 p43 p44 p45 p46 p47 p48 p49 : Prop)
  (p50 p51 p52 p53 p54 p55 p56 p57 p58 p59 : Prop)
  (p60 p61 p62 p63 p64 p65 p66 p67 p68 p69 : Prop) :

  p00 -> p01 -> p02 -> p03 -> p04 -> p05 -> p06 -> p07 -> p08 -> p09 ->
  p10 -> p11 -> p12 -> p13 -> p14 -> p15 -> p16 -> p17 -> p18 -> p19 ->
  p20 -> p21 -> p22 -> p23 -> p24 -> p25 -> p26 -> p27 -> p28 -> p29 ->
  p30 -> p31 -> p32 -> p33 -> p34 -> p35 -> p36 -> p37 -> p38 -> p39 ->
  p40 -> p41 -> p42 -> p43 -> p44 -> p45 -> p46 -> p47 -> p48 -> p49 ->
  p50 -> p51 -> p52 -> p53 -> p54 -> p55 -> p56 -> p57 -> p58 -> p59 -> 
  p60 -> p61 -> p62 -> p63 -> p64 -> p65 -> p66 -> p67 -> p68 -> p69 -> 

  p00 /\ p01 /\ p02 /\ p03 /\ p04 /\ p05 /\ p06 /\ p07 /\ p08 /\ p09 /\
  p10 /\ p11 /\ p12 /\ p13 /\ p14 /\ p15 /\ p16 /\ p17 /\ p18 /\ p19 /\
  p20 /\ p21 /\ p22 /\ p23 /\ p24 /\ p25 /\ p26 /\ p27 /\ p28 /\ p29 /\
  p30 /\ p31 /\ p32 /\ p33 /\ p34 /\ p35 /\ p36 /\ p37 /\ p38 /\ p39 /\
  p40 /\ p41 /\ p42 /\ p43 /\ p44 /\ p45 /\ p46 /\ p47 /\ p48 /\ p49 /\
  p50 /\ p51 /\ p52 /\ p53 /\ p54 /\ p55 /\ p56 /\ p57 /\ p58 /\ p59 /\
  p60 /\ p61 /\ p62 /\ p63 /\ p64 /\ p65 /\ p66 /\ p67 /\ p68 /\ p69.
Proof. 
  idtac "List 70".
  Time refine (run (WithList.tauto _)).
Qed.
